@@ -1,4 +1,4 @@
-from typing import List
+from typing import List, cast
 
 import pytest
 import pyteal as pt
@@ -106,7 +106,7 @@ def test_DynamicArray_decode():
                 assert expr.type_of() == pt.TealType.none
                 assert not expr.has_return()
 
-                expectedExpr = value.stored_value.store(
+                expectedExpr = value._stored_value.store(
                     substring_for_decoding(
                         encoded,
                         start_index=start_index,
@@ -141,7 +141,7 @@ def test_DynamicArray_set_values():
         assert not expr.has_return()
 
         length_tmp = abi.Uint16()
-        expectedExpr = value.stored_value.store(
+        expectedExpr = value._stored_value.store(
             pt.Concat(
                 pt.Seq(length_tmp.set(len(values)), length_tmp.encode()),
                 _encode_tuple(values),
@@ -182,8 +182,16 @@ def test_DynamicArray_set_copy():
 
     expected = pt.TealSimpleBlock(
         [
-            pt.TealOp(None, pt.Op.load, otherArray.stored_value.slot),
-            pt.TealOp(None, pt.Op.store, value.stored_value.slot),
+            pt.TealOp(
+                None,
+                pt.Op.load,
+                cast(pt.ScratchVar, otherArray._stored_value).slot,
+            ),
+            pt.TealOp(
+                None,
+                pt.Op.store,
+                cast(pt.ScratchVar, value._stored_value).slot,
+            ),
         ]
     )
 
@@ -207,7 +215,11 @@ def test_DynamicArray_set_computed():
     expected = pt.TealSimpleBlock(
         [
             pt.TealOp(None, pt.Op.byte, '"this should be a dynamic array"'),
-            pt.TealOp(None, pt.Op.store, value.stored_value.slot),
+            pt.TealOp(
+                None,
+                pt.Op.store,
+                cast(pt.ScratchVar, value._stored_value).slot,
+            ),
         ]
     )
     actual, _ = expr.__teal__(options)
@@ -253,7 +265,11 @@ def test_DynamicBytes_set_py_bytes(test_case: bytes | bytearray):
     expected = pt.TealSimpleBlock(
         [
             pt.TealOp(None, pt.Op.byte, "0x" + length_encoding + BYTE_HEX_TEST_CASE),
-            pt.TealOp(None, pt.Op.store, value.stored_value.slot),
+            pt.TealOp(
+                None,
+                pt.Op.store,
+                cast(pt.ScratchVar, value._stored_value).slot,
+            ),
         ]
     )
 
@@ -280,14 +296,30 @@ def test_DynamicBytes_set_expr(test_case: bytes | bytearray):
             pt.TealOp(None, pt.Op.byte, "0x" + BYTE_HEX_TEST_CASE),
             pt.TealOp(None, pt.Op.byte, "0x" + BYTE_HEX_TEST_CASE),
             pt.TealOp(None, pt.Op.concat),
-            pt.TealOp(None, pt.Op.store, value.stored_value.slot),
-            pt.TealOp(None, pt.Op.load, value.stored_value.slot),
+            pt.TealOp(
+                None,
+                pt.Op.store,
+                cast(pt.ScratchVar, value._stored_value).slot,
+            ),
+            pt.TealOp(
+                None,
+                pt.Op.load,
+                cast(pt.ScratchVar, value._stored_value).slot,
+            ),
             pt.TealOp(None, pt.Op.len),
             pt.TealOp(None, pt.Op.itob),
             pt.TealOp(None, pt.Op.extract, 6, 0),
-            pt.TealOp(None, pt.Op.load, value.stored_value.slot),
+            pt.TealOp(
+                None,
+                pt.Op.load,
+                cast(pt.ScratchVar, value._stored_value).slot,
+            ),
             pt.TealOp(None, pt.Op.concat),
-            pt.TealOp(None, pt.Op.store, value.stored_value.slot),
+            pt.TealOp(
+                None,
+                pt.Op.store,
+                cast(pt.ScratchVar, value._stored_value).slot,
+            ),
         ]
     )
 
@@ -308,7 +340,11 @@ def test_DynamicBytes_get():
 
     expected = pt.TealSimpleBlock(
         [
-            pt.TealOp(None, pt.Op.load, value.stored_value.slot),
+            pt.TealOp(
+                None,
+                pt.Op.load,
+                cast(pt.ScratchVar, value._stored_value).slot,
+            ),
             pt.TealOp(None, pt.Op.extract, 2, 0),
         ]
     )
@@ -325,7 +361,13 @@ def test_DynamicArray_encode():
     assert not expr.has_return()
 
     expected = pt.TealSimpleBlock(
-        [pt.TealOp(None, pt.Op.load, value.stored_value.slot)]
+        [
+            pt.TealOp(
+                None,
+                pt.Op.load,
+                cast(pt.ScratchVar, value._stored_value).slot,
+            ),
+        ]
     )
 
     actual, _ = expr.__teal__(options)
