@@ -1,6 +1,7 @@
 import pytest
 
 import pyteal as pt
+from pyteal.errors import TealInputError
 
 options = pt.CompileOptions()
 
@@ -88,6 +89,15 @@ def test_bytes_base16_empty():
     assert actual == expected
 
 
+B16_ODD_LEN_TESTCASES = ["F", "0c1"]
+
+
+@pytest.mark.parametrize("testcase", B16_ODD_LEN_TESTCASES)
+def test_bytes_base16_odd_len(testcase):
+    with pytest.raises(TealInputError):
+        pt.Bytes("base16", testcase)
+
+
 def test_bytes_utf8():
     expr = pt.Bytes("hello world")
     assert expr.type_of() == pt.TealType.bytes
