@@ -1,13 +1,17 @@
 from itertools import product
-from typing import List, Literal, Optional, cast
+from typing import Callable, Literal, Optional, cast
+from inspect import signature
 
 import pytest
 from dataclasses import dataclass
 
 import pyteal as pt
-from pyteal.ast.subroutine import ABIReturnSubroutine, evaluate_subroutine
+from pyteal.ast.frame import Proto, ProtoStackLayout, FrameBury, FrameDig
+from pyteal.ast.subroutine import ABIReturnSubroutine, SubroutineEval
+from pyteal.compiler.compiler import FRAME_POINTERS_VERSION
 
 options = pt.CompileOptions(version=5)
+options_v8 = pt.CompileOptions(version=8)
 
 
 def test_subroutine_definition():
@@ -1138,172 +1142,701 @@ def test_decorator():
         mySubroutine(a=pt.Int(1))
 
 
-def test_evaluate_subroutine_no_args():
-    cases = (
-        (pt.TealType.none, pt.Return()),
-        (pt.TealType.uint64, pt.Int(1) + pt.Int(2)),
-        (pt.TealType.uint64, pt.Return(pt.Int(1) + pt.Int(2))),
-        (pt.TealType.bytes, pt.Bytes("value")),
-        (pt.TealType.bytes, pt.Return(pt.Bytes("value"))),
-    )
+SUBROUTINE_VAR_ARGS_CASES = [
+    (pt.TealType.none, pt.Return()),
+    (pt.TealType.uint64, pt.Int(1) + pt.Int(2)),
+    (pt.TealType.uint64, pt.Return(pt.Int(1) + pt.Int(2))),
+    (pt.TealType.bytes, pt.Bytes("value")),
+    (pt.TealType.bytes, pt.Return(pt.Bytes("value"))),
+]
 
-    for (returnType, returnValue) in cases:
 
-        def mySubroutine():
-            return returnValue
+@pytest.mark.parametrize("return_type, return_value", SUBROUTINE_VAR_ARGS_CASES)
+def test_evaluate_subroutine_no_args(return_type: pt.TealType, return_value: pt.Expr):
+    def mySubroutine():
+        return return_value
+
+    definition = pt.SubroutineDefinition(mySubroutine, return_type)
+    evaluate_subroutine = SubroutineEval.normal_evaluator()
+
+    declaration = evaluate_subroutine(definition)
+
+    assert isinstance(declaration, pt.SubroutineDeclaration)
+    assert declaration.subroutine is definition
+
+    assert declaration.type_of() == return_value.type_of()
+    assert declaration.has_return() == return_value.has_return()
+
+    options.setSubroutine(definition)
+    expected, _ = pt.Seq([return_value]).__teal__(options)
+
+    actual, _ = declaration.__teal__(options)
+    options.setSubroutine(None)
+    assert actual == expected
+
+
+@pytest.mark.parametrize("return_type, return_value", SUBROUTINE_VAR_ARGS_CASES)
+def test_evaluate_subroutine_1_arg(return_type: pt.TealType, return_value: pt.Expr):
+    argSlots: list[pt.ScratchSlot] = []
+
+    def mySubroutine(a1):
+        assert isinstance(a1, pt.ScratchLoad)
+        argSlots.append(a1.slot)
+        return return_value
+
+    definition = pt.SubroutineDefinition(mySubroutine, return_type)
+
+    evaluate_subroutine = SubroutineEval.normal_evaluator()
+    declaration = evaluate_subroutine(definition)
+
+    assert isinstance(declaration, pt.SubroutineDeclaration)
+    assert declaration.subroutine is definition
+
+    assert declaration.type_of() == return_value.type_of()
+    assert declaration.has_return() == return_value.has_return()
+
+    assert isinstance(declaration.body, pt.Seq)
+    assert len(declaration.body.args) == 2
+
+    assert isinstance(declaration.body.args[0], pt.ScratchStackStore)
+
+    assert declaration.body.args[0].slot is argSlots[-1]
+
+    options.setSubroutine(definition)
+    expected, _ = pt.Seq([declaration.body.args[0], return_value]).__teal__(options)
+
+    actual, _ = declaration.__teal__(options)
+    options.setSubroutine(None)
+    assert actual == expected
+
+
+@pytest.mark.parametrize("return_type, return_value", SUBROUTINE_VAR_ARGS_CASES)
+def test_evaluate_subroutine_2_args(return_type: pt.TealType, return_value: pt.Expr):
+    argSlots: list[pt.ScratchSlot] = []
+
+    def mySubroutine(a1, a2):
+        assert isinstance(a1, pt.ScratchLoad)
+        argSlots.append(a1.slot)
+        assert isinstance(a2, pt.ScratchLoad)
+        argSlots.append(a2.slot)
+        return return_value
+
+    definition = pt.SubroutineDefinition(mySubroutine, return_type)
+
+    evaluate_subroutine = SubroutineEval.normal_evaluator()
+    declaration = evaluate_subroutine(definition)
+
+    assert isinstance(declaration, pt.SubroutineDeclaration)
+    assert declaration.subroutine is definition
+
+    assert declaration.type_of() == return_value.type_of()
+    assert declaration.has_return() == return_value.has_return()
+
+    assert isinstance(declaration.body, pt.Seq)
+    assert len(declaration.body.args) == 3
+
+    assert isinstance(declaration.body.args[0], pt.ScratchStackStore)
+    assert isinstance(declaration.body.args[1], pt.ScratchStackStore)
+
+    assert declaration.body.args[0].slot is argSlots[-1]
+    assert declaration.body.args[1].slot is argSlots[-2]
+
+    options.setSubroutine(definition)
+    expected, _ = pt.Seq(
+        [declaration.body.args[0], declaration.body.args[1], return_value]
+    ).__teal__(options)
+
+    actual, _ = declaration.__teal__(options)
+    options.setSubroutine(None)
+    assert actual == expected
+
+
+@pytest.mark.parametrize("return_type, return_value", SUBROUTINE_VAR_ARGS_CASES)
+def test_evaluate_subroutine_10_args(return_type: pt.TealType, return_value: pt.Expr):
+    argSlots: list[pt.ScratchSlot] = []
+
+    def mySubroutine(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10):
+        for a in (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10):
+            assert isinstance(a, pt.ScratchLoad)
+            argSlots.append(a.slot)
+        return return_value
+
+    definition = pt.SubroutineDefinition(mySubroutine, return_type)
+
+    evaluate_subroutine = SubroutineEval.normal_evaluator()
+    declaration = evaluate_subroutine(definition)
+
+    assert isinstance(declaration, pt.SubroutineDeclaration)
+    assert declaration.subroutine is definition
+
+    assert declaration.type_of() == return_value.type_of()
+    assert declaration.has_return() == return_value.has_return()
+
+    assert isinstance(declaration.body, pt.Seq)
+    assert len(declaration.body.args) == 11
+
+    for i in range(10):
+        assert isinstance(declaration.body.args[i], pt.ScratchStackStore)
+
+    for i in range(10):
+        assert declaration.body.args[i].slot is argSlots[-i - 1]
+
+    options.setSubroutine(definition)
+    expected, _ = pt.Seq(declaration.body.args[:10] + [return_value]).__teal__(options)
+
+    actual, _ = declaration.__teal__(options)
+    options.setSubroutine(None)
+    assert actual == expected
+
+
+@pytest.mark.parametrize("return_type, return_value", SUBROUTINE_VAR_ARGS_CASES)
+def test_evaluate_subroutine_frame_pt_version(
+    return_type: pt.TealType, return_value: pt.Expr
+):
+    def mySubroutine_no_arg():
+        return return_value
+
+    def mySubroutine_arg_1(a1):
+        return return_value
+
+    def mySubroutine_arg_2(a1, a2):
+        return return_value
+
+    def mySubroutine_arg_10(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10):
+        return return_value
+
+    for subr in [
+        mySubroutine_no_arg,
+        mySubroutine_arg_1,
+        mySubroutine_arg_2,
+        mySubroutine_arg_10,
+    ]:
+        subr = cast(Callable[..., pt.Expr], subr)
+        definition = pt.SubroutineDefinition(subr, return_type)
+        evaluate_subroutine = SubroutineEval.fp_evaluator()
 
-        definition = pt.SubroutineDefinition(mySubroutine, returnType)
         declaration = evaluate_subroutine(definition)
 
         assert isinstance(declaration, pt.SubroutineDeclaration)
         assert declaration.subroutine is definition
 
-        assert declaration.type_of() == returnValue.type_of()
-        assert declaration.has_return() == returnValue.has_return()
-
-        options.setSubroutine(definition)
-        expected, _ = pt.Seq([returnValue]).__teal__(options)
-
-        actual, _ = declaration.__teal__(options)
-        options.setSubroutine(None)
-        assert actual == expected
-
-
-def test_evaluate_subroutine_1_arg():
-    cases = (
-        (pt.TealType.none, pt.Return()),
-        (pt.TealType.uint64, pt.Int(1) + pt.Int(2)),
-        (pt.TealType.uint64, pt.Return(pt.Int(1) + pt.Int(2))),
-        (pt.TealType.bytes, pt.Bytes("value")),
-        (pt.TealType.bytes, pt.Return(pt.Bytes("value"))),
-    )
-
-    for (returnType, returnValue) in cases:
-        argSlots: List[pt.ScratchSlot] = []
-
-        def mySubroutine(a1):
-            assert isinstance(a1, pt.ScratchLoad)
-            argSlots.append(a1.slot)
-            return returnValue
-
-        definition = pt.SubroutineDefinition(mySubroutine, returnType)
-        declaration = evaluate_subroutine(definition)
-
-        assert isinstance(declaration, pt.SubroutineDeclaration)
-        assert declaration.subroutine is definition
-
-        assert declaration.type_of() == returnValue.type_of()
-        assert declaration.has_return() == returnValue.has_return()
+        assert declaration.type_of() == return_value.type_of()
+        assert declaration.has_return() == return_value.has_return()
 
         assert isinstance(declaration.body, pt.Seq)
         assert len(declaration.body.args) == 2
 
-        assert isinstance(declaration.body.args[0], pt.ScratchStackStore)
+        assert isinstance(declaration.body.args[0], Proto)
 
-        assert declaration.body.args[0].slot is argSlots[-1]
+        proto_expr = declaration.body.args[0]
+        assert proto_expr.num_returns == int(return_type != pt.TealType.none)
+        assert proto_expr.num_args == len(signature(subr).parameters)
 
-        options.setSubroutine(definition)
-        expected, _ = pt.Seq([declaration.body.args[0], returnValue]).__teal__(options)
+        assert isinstance(declaration.body.args[1], type(return_value))
 
-        actual, _ = declaration.__teal__(options)
-        options.setSubroutine(None)
-        assert actual == expected
-
-
-def test_evaluate_subroutine_2_args():
-    cases = (
-        (pt.TealType.none, pt.Return()),
-        (pt.TealType.uint64, pt.Int(1) + pt.Int(2)),
-        (pt.TealType.uint64, pt.Return(pt.Int(1) + pt.Int(2))),
-        (pt.TealType.bytes, pt.Bytes("value")),
-        (pt.TealType.bytes, pt.Return(pt.Bytes("value"))),
-    )
-
-    for (returnType, returnValue) in cases:
-        argSlots: List[pt.ScratchSlot] = []
-
-        def mySubroutine(a1, a2):
-            assert isinstance(a1, pt.ScratchLoad)
-            argSlots.append(a1.slot)
-            assert isinstance(a2, pt.ScratchLoad)
-            argSlots.append(a2.slot)
-            return returnValue
-
-        definition = pt.SubroutineDefinition(mySubroutine, returnType)
-
-        declaration = evaluate_subroutine(definition)
-
-        assert isinstance(declaration, pt.SubroutineDeclaration)
-        assert declaration.subroutine is definition
-
-        assert declaration.type_of() == returnValue.type_of()
-        assert declaration.has_return() == returnValue.has_return()
-
-        assert isinstance(declaration.body, pt.Seq)
-        assert len(declaration.body.args) == 3
-
-        assert isinstance(declaration.body.args[0], pt.ScratchStackStore)
-        assert isinstance(declaration.body.args[1], pt.ScratchStackStore)
-
-        assert declaration.body.args[0].slot is argSlots[-1]
-        assert declaration.body.args[1].slot is argSlots[-2]
-
-        options.setSubroutine(definition)
-        expected, _ = pt.Seq(
-            [declaration.body.args[0], declaration.body.args[1], returnValue]
-        ).__teal__(options)
-
-        actual, _ = declaration.__teal__(options)
-        options.setSubroutine(None)
-        assert actual == expected
-
-
-def test_evaluate_subroutine_10_args():
-    cases = (
-        (pt.TealType.none, pt.Return()),
-        (pt.TealType.uint64, pt.Int(1) + pt.Int(2)),
-        (pt.TealType.uint64, pt.Return(pt.Int(1) + pt.Int(2))),
-        (pt.TealType.bytes, pt.Bytes("value")),
-        (pt.TealType.bytes, pt.Return(pt.Bytes("value"))),
-    )
-
-    for (returnType, returnValue) in cases:
-        argSlots: List[pt.ScratchSlot] = []
-
-        def mySubroutine(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10):
-            for a in (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10):
-                assert isinstance(a, pt.ScratchLoad)
-                argSlots.append(a.slot)
-            return returnValue
-
-        definition = pt.SubroutineDefinition(mySubroutine, returnType)
-        declaration = evaluate_subroutine(definition)
-
-        assert isinstance(declaration, pt.SubroutineDeclaration)
-        assert declaration.subroutine is definition
-
-        assert declaration.type_of() == returnValue.type_of()
-        assert declaration.has_return() == returnValue.has_return()
-
-        assert isinstance(declaration.body, pt.Seq)
-        assert len(declaration.body.args) == 11
-
-        for i in range(10):
-            assert isinstance(declaration.body.args[i], pt.ScratchStackStore)
-
-        for i in range(10):
-            assert declaration.body.args[i].slot is argSlots[-i - 1]
-
-        options.setSubroutine(definition)
-        expected, _ = pt.Seq(declaration.body.args[:10] + [returnValue]).__teal__(
-            options
+        options_v8.setSubroutine(definition)
+        expected, _ = pt.Seq([declaration.body.args[0], return_value]).__teal__(
+            options_v8
         )
 
-        actual, _ = declaration.__teal__(options)
-        options.setSubroutine(None)
+        actual, _ = declaration.__teal__(options_v8)
+        options_v8.setSubroutine(None)
         assert actual == expected
 
 
+@dataclass
+class LocalVariableTestCase:
+    input_subroutine: Callable[..., pt.Expr]
+    input_subroutine_return_type: pt.TealType
+    input_subroutine_abi_return: bool
+    expected_body_normal_evaluator: pt.Expr
+    expected_body_fp_evaluator: pt.Expr
+
+
+def example_subroutine_no_args_no_return():
+    local_scratch_var = pt.ScratchVar(pt.TealType.uint64)
+    local_abi_type = pt.abi.Uint64()
+    return pt.Seq(
+        local_scratch_var.store(pt.Int(1)),
+        local_abi_type.set(pt.Int(2)),
+    )
+
+
+def example_subroutine_no_args_uint64_return():
+    local_scratch_var = pt.ScratchVar(pt.TealType.uint64)
+    local_abi_type = pt.abi.Uint64()
+    return pt.Seq(
+        local_scratch_var.store(pt.Int(1)), local_abi_type.set(pt.Int(2)), pt.Int(3)
+    )
+
+
+def example_subroutine_no_args_bytes_return():
+    local_scratch_var = pt.ScratchVar(pt.TealType.uint64)
+    local_abi_type = pt.abi.Uint64()
+    return pt.Seq(
+        local_scratch_var.store(pt.Int(1)),
+        local_abi_type.set(pt.Int(2)),
+        pt.Bytes(b"abc"),
+    )
+
+
+def example_subroutine_no_args_abi_return(*, output: pt.abi.Uint64):
+    local_scratch_var = pt.ScratchVar(pt.TealType.uint64)
+    local_abi_type = pt.abi.Uint64()
+    return pt.Seq(
+        local_scratch_var.store(pt.Int(1)),
+        local_abi_type.set(pt.Int(2)),
+        output.set(pt.Int(3)),
+    )
+
+
+def example_subroutine_expr_args_uint64_return(a: pt.Expr, b: pt.Expr):
+    local_scratch_var = pt.ScratchVar(pt.TealType.uint64)
+    local_abi_type = pt.abi.Uint64()
+    return pt.Seq(
+        pt.Pop(a == pt.Len(b)),
+        local_scratch_var.store(pt.Int(1)),
+        local_abi_type.set(pt.Int(2)),
+        pt.Int(3),
+    )
+
+
+def example_subroutine_expr_args_bytes_return(a: pt.Expr, b: pt.Expr):
+    local_scratch_var = pt.ScratchVar(pt.TealType.uint64)
+    local_abi_type = pt.abi.Uint64()
+    return pt.Seq(
+        pt.Pop(a == pt.Len(b)),
+        local_scratch_var.store(pt.Int(1)),
+        local_abi_type.set(pt.Int(2)),
+        pt.Bytes(b"abc"),
+    )
+
+
+def example_subroutine_expr_args_abi_return(
+    a: pt.Expr, b: pt.Expr, *, output: pt.abi.StaticBytes[Literal[5]]
+):
+    local_scratch_var = pt.ScratchVar(pt.TealType.uint64)
+    local_abi_type = pt.abi.Uint64()
+    return pt.Seq(
+        pt.Pop(a == pt.Len(b)),
+        local_scratch_var.store(pt.Int(1)),
+        local_abi_type.set(pt.Int(2)),
+        output.set(b"hello"),
+    )
+
+
+def example_subroutine_abi_args_uint64_return(a: pt.abi.Uint8, b: pt.abi.String):
+    local_scratch_var = pt.ScratchVar(pt.TealType.uint64)
+    local_abi_type = pt.abi.Uint64()
+    return pt.Seq(
+        pt.Pop(a.get() == pt.Len(b.get())),
+        local_scratch_var.store(pt.Int(1)),
+        local_abi_type.set(pt.Int(2)),
+        pt.Int(3),
+    )
+
+
+def example_subroutine_abi_args_bytes_return(a: pt.abi.Uint8, b: pt.abi.String):
+    local_scratch_var = pt.ScratchVar(pt.TealType.uint64)
+    local_abi_type = pt.abi.Uint64()
+    return pt.Seq(
+        pt.Pop(a.get() == pt.Len(b.get())),
+        local_scratch_var.store(pt.Int(1)),
+        local_abi_type.set(pt.Int(2)),
+        pt.Bytes(b"abc"),
+    )
+
+
+def example_subroutine_abi_args_abi_return(
+    a: pt.abi.Uint8, b: pt.abi.String, *, output: pt.abi.StaticBytes[Literal[5]]
+):
+    local_scratch_var = pt.ScratchVar(pt.TealType.uint64)
+    local_abi_type = pt.abi.Uint64()
+    return pt.Seq(
+        pt.Pop(a.get() == pt.Len(b.get())),
+        local_scratch_var.store(pt.Int(1)),
+        local_abi_type.set(pt.Int(2)),
+        output.set(b"hello"),
+    )
+
+
+def example_subroutine_many_local_vars():
+    local_abi_vars = [pt.abi.Uint64() for _ in range(200)]
+    return pt.Seq([v.set(pt.Int(i)) for i, v in enumerate(local_abi_vars)])
+
+
+@pytest.mark.parametrize(
+    "test_case",
+    [
+        LocalVariableTestCase(
+            input_subroutine=example_subroutine_no_args_no_return,
+            input_subroutine_return_type=pt.TealType.none,
+            input_subroutine_abi_return=False,
+            expected_body_normal_evaluator=pt.Seq(
+                pt.ScratchVar().store(pt.Int(1)),
+                pt.abi.Uint64().set(pt.Int(2)),
+            ),
+            expected_body_fp_evaluator=pt.Seq(
+                Proto(
+                    0,
+                    0,
+                    mem_layout=ProtoStackLayout(
+                        arg_stack_types=[],
+                        local_stack_types=[pt.TealType.uint64],
+                        num_return_allocs=0,
+                    ),
+                ),
+                pt.ScratchVar().store(pt.Int(1)),
+                FrameBury(pt.Int(2), 0),
+            ),
+        ),
+        LocalVariableTestCase(
+            input_subroutine=example_subroutine_no_args_uint64_return,
+            input_subroutine_return_type=pt.TealType.uint64,
+            input_subroutine_abi_return=False,
+            expected_body_normal_evaluator=pt.Seq(
+                pt.ScratchVar().store(pt.Int(1)),
+                pt.abi.Uint64().set(pt.Int(2)),
+                pt.Int(3),
+            ),
+            expected_body_fp_evaluator=pt.Seq(
+                Proto(
+                    0,
+                    1,
+                    mem_layout=ProtoStackLayout(
+                        arg_stack_types=[],
+                        local_stack_types=[pt.TealType.uint64],
+                        num_return_allocs=0,
+                    ),
+                ),
+                pt.ScratchVar(pt.TealType.uint64).store(pt.Int(1)),
+                FrameBury(pt.Int(2), 0),
+                # overwrite 1st local variable with the return value
+                FrameBury(pt.Int(3), 0),
+            ),
+        ),
+        LocalVariableTestCase(
+            input_subroutine=example_subroutine_no_args_bytes_return,
+            input_subroutine_return_type=pt.TealType.bytes,
+            input_subroutine_abi_return=False,
+            expected_body_normal_evaluator=pt.Seq(
+                pt.ScratchVar().store(pt.Int(1)),
+                pt.abi.Uint64().set(pt.Int(2)),
+                pt.Bytes(b"abc"),
+            ),
+            expected_body_fp_evaluator=pt.Seq(
+                Proto(
+                    0,
+                    1,
+                    mem_layout=ProtoStackLayout(
+                        arg_stack_types=[],
+                        local_stack_types=[pt.TealType.uint64],
+                        num_return_allocs=0,
+                    ),
+                ),
+                pt.ScratchVar(pt.TealType.uint64).store(pt.Int(1)),
+                FrameBury(pt.Int(2), 0),
+                # overwrite 1st local variable with the return value
+                FrameBury(pt.Bytes(b"abc"), 0),
+            ),
+        ),
+        LocalVariableTestCase(
+            input_subroutine=example_subroutine_no_args_abi_return,
+            input_subroutine_return_type=pt.TealType.none,
+            input_subroutine_abi_return=True,
+            expected_body_normal_evaluator=pt.Seq(
+                pt.ScratchVar().store(pt.Int(1)),
+                pt.abi.Uint64().set(pt.Int(2)),
+                (output_uint64 := pt.abi.Uint64()).set(pt.Int(3)),
+                output_uint64.get(),
+            ),
+            expected_body_fp_evaluator=pt.Seq(
+                Proto(
+                    0,
+                    1,
+                    mem_layout=ProtoStackLayout(
+                        arg_stack_types=[],
+                        local_stack_types=[pt.TealType.uint64, pt.TealType.uint64],
+                        num_return_allocs=1,
+                    ),
+                ),
+                pt.ScratchVar(pt.TealType.uint64).store(pt.Int(1)),
+                FrameBury(pt.Int(2), 1),
+                FrameBury(pt.Int(3), 0),
+            ),
+        ),
+        LocalVariableTestCase(
+            input_subroutine=example_subroutine_expr_args_uint64_return,
+            input_subroutine_return_type=pt.TealType.uint64,
+            input_subroutine_abi_return=False,
+            expected_body_normal_evaluator=pt.Seq(
+                (arg2_expr := pt.ScratchVar()).slot.store(),
+                (arg1_expr := pt.ScratchVar()).slot.store(),
+                pt.Pop(arg1_expr.load() == pt.Len(arg2_expr.load())),
+                pt.ScratchVar().store(pt.Int(1)),
+                pt.abi.Uint64().set(pt.Int(2)),
+                pt.Int(3),
+            ),
+            expected_body_fp_evaluator=pt.Seq(
+                Proto(
+                    2,
+                    1,
+                    mem_layout=ProtoStackLayout(
+                        arg_stack_types=[pt.TealType.anytype, pt.TealType.anytype],
+                        local_stack_types=[pt.TealType.uint64],
+                        num_return_allocs=0,
+                    ),
+                ),
+                pt.Pop(FrameDig(-2) == pt.Len(FrameDig(-1))),
+                pt.ScratchVar(pt.TealType.uint64).store(pt.Int(1)),
+                FrameBury(pt.Int(2), 0),
+                # overwrite 1st local variable with the return value
+                FrameBury(pt.Int(3), 0),
+            ),
+        ),
+        LocalVariableTestCase(
+            input_subroutine=example_subroutine_expr_args_bytes_return,
+            input_subroutine_return_type=pt.TealType.bytes,
+            input_subroutine_abi_return=False,
+            expected_body_normal_evaluator=pt.Seq(
+                (arg2_expr := pt.ScratchVar()).slot.store(),
+                (arg1_expr := pt.ScratchVar()).slot.store(),
+                pt.Pop(arg1_expr.load() == pt.Len(arg2_expr.load())),
+                pt.ScratchVar().store(pt.Int(1)),
+                pt.abi.Uint64().set(pt.Int(2)),
+                pt.Bytes(b"abc"),
+            ),
+            expected_body_fp_evaluator=pt.Seq(
+                Proto(
+                    2,
+                    1,
+                    mem_layout=ProtoStackLayout(
+                        arg_stack_types=[pt.TealType.anytype, pt.TealType.anytype],
+                        local_stack_types=[pt.TealType.uint64],
+                        num_return_allocs=0,
+                    ),
+                ),
+                pt.Pop(FrameDig(-2) == pt.Len(FrameDig(-1))),
+                pt.ScratchVar(pt.TealType.uint64).store(pt.Int(1)),
+                FrameBury(pt.Int(2), 0),
+                # overwrite 1st local variable with the return value
+                FrameBury(pt.Bytes(b"abc"), 0),
+            ),
+        ),
+        LocalVariableTestCase(
+            input_subroutine=example_subroutine_expr_args_abi_return,
+            input_subroutine_return_type=pt.TealType.none,
+            input_subroutine_abi_return=True,
+            expected_body_normal_evaluator=pt.Seq(
+                (arg2_expr := pt.ScratchVar()).slot.store(),
+                (arg1_expr := pt.ScratchVar()).slot.store(),
+                pt.Pop(arg1_expr.load() == pt.Len(arg2_expr.load())),
+                pt.ScratchVar().store(pt.Int(1)),
+                pt.abi.Uint64().set(pt.Int(2)),
+                (
+                    output_static_bytes := pt.abi.make(pt.abi.StaticBytes[Literal[5]])
+                ).set(b"hello"),
+                output_static_bytes.get(),
+            ),
+            expected_body_fp_evaluator=pt.Seq(
+                Proto(
+                    2,
+                    1,
+                    mem_layout=ProtoStackLayout(
+                        arg_stack_types=[pt.TealType.anytype, pt.TealType.anytype],
+                        local_stack_types=[pt.TealType.bytes, pt.TealType.uint64],
+                        num_return_allocs=1,
+                    ),
+                ),
+                pt.Pop(FrameDig(-2) == pt.Len(FrameDig(-1))),
+                pt.ScratchVar(pt.TealType.uint64).store(pt.Int(1)),
+                FrameBury(pt.Int(2), 1),
+                FrameBury(pt.Bytes(b"hello"), 0),
+            ),
+        ),
+        LocalVariableTestCase(
+            input_subroutine=example_subroutine_abi_args_uint64_return,
+            input_subroutine_return_type=pt.TealType.uint64,
+            input_subroutine_abi_return=False,
+            expected_body_normal_evaluator=pt.Seq(
+                cast(
+                    pt.ScratchVar, (arg_string := pt.abi.String())._stored_value
+                ).slot.store(),
+                cast(
+                    pt.ScratchVar, (arg_uint8 := pt.abi.Uint8())._stored_value
+                ).slot.store(),
+                pt.Pop(arg_uint8.get() == pt.Len(arg_string.get())),
+                pt.ScratchVar().store(pt.Int(1)),
+                pt.abi.Uint64().set(pt.Int(2)),
+                pt.Int(3),
+            ),
+            expected_body_fp_evaluator=pt.Seq(
+                Proto(
+                    2,
+                    1,
+                    mem_layout=ProtoStackLayout(
+                        arg_stack_types=[pt.TealType.uint64, pt.TealType.bytes],
+                        local_stack_types=[pt.TealType.uint64],
+                        num_return_allocs=0,
+                    ),
+                ),
+                pt.Pop(FrameDig(-2) == pt.Len(pt.Suffix(FrameDig(-1), pt.Int(2)))),
+                pt.ScratchVar(pt.TealType.uint64).store(pt.Int(1)),
+                FrameBury(pt.Int(2), 0),
+                # overwrite 1st local variable with the return value
+                FrameBury(pt.Int(3), 0),
+            ),
+        ),
+        LocalVariableTestCase(
+            input_subroutine=example_subroutine_abi_args_bytes_return,
+            input_subroutine_return_type=pt.TealType.bytes,
+            input_subroutine_abi_return=False,
+            expected_body_normal_evaluator=pt.Seq(
+                cast(
+                    pt.ScratchVar, (arg_string := pt.abi.String())._stored_value
+                ).slot.store(),
+                cast(
+                    pt.ScratchVar, (arg_uint8 := pt.abi.Uint8())._stored_value
+                ).slot.store(),
+                pt.Pop(arg_uint8.get() == pt.Len(arg_string.get())),
+                pt.ScratchVar().store(pt.Int(1)),
+                pt.abi.Uint64().set(pt.Int(2)),
+                pt.Bytes(b"abc"),
+            ),
+            expected_body_fp_evaluator=pt.Seq(
+                Proto(
+                    2,
+                    1,
+                    mem_layout=ProtoStackLayout(
+                        arg_stack_types=[pt.TealType.uint64, pt.TealType.bytes],
+                        local_stack_types=[pt.TealType.uint64],
+                        num_return_allocs=0,
+                    ),
+                ),
+                pt.Pop(FrameDig(-2) == pt.Len(pt.Suffix(FrameDig(-1), pt.Int(2)))),
+                pt.ScratchVar(pt.TealType.uint64).store(pt.Int(1)),
+                FrameBury(pt.Int(2), 0),
+                # overwrite 1st local variable with the return value
+                FrameBury(pt.Bytes(b"abc"), 0),
+            ),
+        ),
+        LocalVariableTestCase(
+            input_subroutine=example_subroutine_abi_args_abi_return,
+            input_subroutine_return_type=pt.TealType.none,
+            input_subroutine_abi_return=True,
+            expected_body_normal_evaluator=pt.Seq(
+                cast(
+                    pt.ScratchVar, (arg_string := pt.abi.String())._stored_value
+                ).slot.store(),
+                cast(
+                    pt.ScratchVar, (arg_uint8 := pt.abi.Uint8())._stored_value
+                ).slot.store(),
+                pt.Pop(arg_uint8.get() == pt.Len(arg_string.get())),
+                pt.ScratchVar().store(pt.Int(1)),
+                pt.abi.Uint64().set(pt.Int(2)),
+                (
+                    output_static_bytes := pt.abi.make(pt.abi.StaticBytes[Literal[5]])
+                ).set(b"hello"),
+                output_static_bytes.get(),
+            ),
+            expected_body_fp_evaluator=pt.Seq(
+                Proto(
+                    2,
+                    1,
+                    mem_layout=ProtoStackLayout(
+                        arg_stack_types=[pt.TealType.uint64, pt.TealType.bytes],
+                        local_stack_types=[pt.TealType.bytes, pt.TealType.uint64],
+                        num_return_allocs=1,
+                    ),
+                ),
+                pt.Pop(FrameDig(-2) == pt.Len(pt.Suffix(FrameDig(-1), pt.Int(2)))),
+                pt.ScratchVar(pt.TealType.uint64).store(pt.Int(1)),
+                FrameBury(pt.Int(2), 1),
+                FrameBury(pt.Bytes(b"hello"), 0),
+            ),
+        ),
+        LocalVariableTestCase(
+            input_subroutine=example_subroutine_many_local_vars,
+            input_subroutine_return_type=pt.TealType.none,
+            input_subroutine_abi_return=False,
+            expected_body_normal_evaluator=pt.Seq(
+                [pt.ScratchVar().store(pt.Int(i)) for i in range(200)]
+            ),
+            expected_body_fp_evaluator=pt.Seq(
+                Proto(
+                    0,
+                    0,
+                    mem_layout=ProtoStackLayout(
+                        arg_stack_types=[],
+                        local_stack_types=[pt.TealType.uint64] * 128,
+                        num_return_allocs=0,
+                    ),
+                ),
+                # 128 is the max number of frame pointer local+return vars
+                *[FrameBury(pt.Int(i), i) for i in range(128)],
+                *[pt.ScratchVar().store(pt.Int(i)) for i in range(128, 200)],
+            ),
+        ),
+    ],
+)
+def test_evaluate_subroutine_local_variables(test_case: LocalVariableTestCase):
+    definition = pt.SubroutineDefinition(
+        test_case.input_subroutine,
+        test_case.input_subroutine_return_type,
+        has_abi_output=test_case.input_subroutine_abi_return,
+    )
+
+    for evaluator, expected_body in (
+        (SubroutineEval.normal_evaluator(), test_case.expected_body_normal_evaluator),
+        (SubroutineEval.fp_evaluator(), test_case.expected_body_fp_evaluator),
+    ):
+        declaration = evaluator(definition)
+
+        evaluator_type = "fp" if evaluator.use_frame_pt else "normal"
+        failure_msg = f"assertion failed for {evaluator_type} evaluator"
+
+        assert isinstance(declaration, pt.SubroutineDeclaration), failure_msg
+        assert declaration.subroutine is definition, failure_msg
+
+        assert (
+            declaration.type_of() is test_case.input_subroutine_return_type
+        ), failure_msg
+        assert declaration.has_return() is False, failure_msg
+
+        options_v8.setSubroutine(definition)
+
+        expected, _ = expected_body.__teal__(options_v8)
+
+        actual, actual_end = declaration.__teal__(options_v8)
+        if declaration.deferred_expr is not None:
+            # This is a hacky way to include the deferred expression in the resulting IR. It's only
+            # valid if there are no retsub opcodes anywhere in the subroutine.
+
+            for block in pt.TealBlock.Iterate(actual):
+                assert all(
+                    op.op != pt.Op.retsub for op in block.ops
+                ), "retsub present in subroutine, test code to apply deferred expression is no longer valid"
+
+            deferred, _ = declaration.deferred_expr.__teal__(options_v8)
+            actual_end.setNextBlock(deferred)
+
+        options_v8.setSubroutine(None)
+
+        expected.addIncoming()
+        expected = pt.TealBlock.NormalizeBlocks(expected)
+
+        actual.addIncoming()
+        actual = pt.TealBlock.NormalizeBlocks(actual)
+
+        with pt.TealComponent.Context.ignoreExprEquality(), pt.TealComponent.Context.ignoreScratchSlotEquality():
+            assert actual == expected, failure_msg
+
+        assert pt.TealBlock.MatchScratchSlotReferences(
+            pt.TealBlock.GetReferencedScratchSlots(actual),
+            pt.TealBlock.GetReferencedScratchSlots(expected),
+        ), failure_msg
+
+
 def test_docstring_parsing_with_different_format():
     short_desc = "Example of a ABIReturnSubroutine with short description docstring."
     a_doc = "an abi Uint64 value"
@@ -1505,3 +2038,9 @@ def test_override_abi_method_name():
 
     mspec = abi_meth_2.method_spec().dictify()
     assert mspec["name"] == "overriden_add"
+
+
+def test_frame_option_version_range_well_formed():
+    assert (
+        pt.Op.callsub.min_version < FRAME_POINTERS_VERSION < pt.MAX_PROGRAM_VERSION + 1
+    )
