@@ -4,13 +4,13 @@ from typing import Union, List, Dict, cast
 from collections import OrderedDict
 from algosdk import encoding
 
-from ..ir import (
+from pyteal.ir import (
     Op,
     TealOp,
     TealComponent,
 )
-from ..util import unescapeStr, correctBase32Padding
-from ..errors import TealInternalError
+from pyteal.util import unescapeStr, correctBase32Padding
+from pyteal.errors import TealInternalError
 
 intEnumValues = {
     # OnComplete values
@@ -104,7 +104,7 @@ def extractMethodSigValue(op: TealOp) -> bytes:
         methodSignature = methodSignature[1:-1]
     else:
         raise TealInternalError(
-            "Method signature opcode error: signatue {} not wrapped with double-quotes".format(
+            "Method signature opcode error: signature {} not wrapped with double-quotes".format(
                 methodSignature
             )
         )
